/*
 * AstriMonitor.cpp
 *
 *  Created on: Jul 3, 2021
 *      Author: astrisw
 */



#include <Astri_Monitor.h>


AstriMonitor::AstriMonitor(){
	reset();
}

AstriMonitor::AstriMonitor(bool totPacket, bool monitLost, bool monitInvalid, bool monitTime, bool monitByte){
	this->monitTotPacketsFlag=totPacket;
	this->monitPacketLostFlag=monitLost;
	this->monitInvalidPacketsFlag=monitInvalid;
	this->monitTimeFlag = monitTime;
	this->monitByteFlag = monitByte;
	reset();
}


void AstriMonitor::printStats(){
	std::string ret = "------------------------------\n";
	if(monitTotPacketsFlag)
		ret = ret.append("|	Tot Packet: ").append(std::to_string(num_pack_rcv)).append("\n");
	if(monitPacketLostFlag)
		ret = ret.append("|	Num Packet lost: ").append(std::to_string(num_pack_lost)).append("\n");
	if(monitTimeFlag){
		if(monitByteFlag){
			double numMB = (double)num_byte_rcv/1000000;
			ret = ret.append("|	Tot MByte received: ").append(std::to_string(numMB)).append("\n");
			ret = ret.append("|	ByteRate: ").append(std::to_string(numMB/(timeElps.count()/1000))).append(" MBs\n");
			ret = ret.append("|	Packet Rate: ").append(std::to_string(num_pack_rcv/(timeElps.count()/1000))).append(" packets\\seconds\n");
		}
		ret = ret.append("|	Time elapsed: ").append(std::to_string(timeElps.count()/1000)).append(" seconds\n");
	}
	if(monitInvalidPacketsFlag)
		ret = ret.append("|	Num Packet invalid: ").append(std::to_string(num_pack_invalid)).append("\n");
	ret = ret.append("------------------------------\n");

	std::cout << ret << std::endl;
}


void AstriMonitor::reset(){
	num_pack_rcv = 0;
	num_pack_lost = 0;
	num_byte_rcv = 0;
	timeElps = std::chrono::steady_clock::duration::zero();
	startTime = std::chrono::steady_clock::time_point::min();
	num_pack_invalid = 0;
}


void AstriMonitor::monit(BasePacket &pack){
	if(monitTotPacketsFlag)
		monitTotPackets();
	if(monitByteFlag)
		monitByte(pack);
	if(monitPacketLostFlag)
		monitPacketLost(pack);
	if(monitInvalidPacketsFlag)
		monitInvalidPackets(pack);
	if(monitTimeFlag)
		monitTime();
}

void AstriMonitor::monitTime(){
    using std::chrono::steady_clock;

    if(startTime == steady_clock::time_point::min()){
    	startTime = steady_clock::now();
    }
    else{
    	timeElps = steady_clock::now() - startTime;
    }
}


void AstriMonitor::monitTotPackets(){
	num_pack_rcv += 1;
}

void AstriMonitor::monitByte(BasePacket & pack){
	num_byte_rcv += pack.getPacketSize();
}

void AstriMonitor::monitPacketLost(BasePacket & pack){
	int curr_ssc = pack.getSSC();
	std::string type = std::to_string( (*pack.getHeaderFieldValues())["Packet Type"]);
	std::string subtype = std::to_string( (*pack.getHeaderFieldValues())["Packet SubType"]);
	std::string entry = type.append(subtype);

	try{
		int last_ssc = sscMap.at(entry);
		if(curr_ssc > last_ssc+1){
			num_pack_lost += curr_ssc - last_ssc -1;
		}
		else
			if(curr_ssc < last_ssc){
				num_pack_lost += 65535 - last_ssc + curr_ssc;
			}
	}
	catch(const std::out_of_range& oor){
		num_pack_lost += curr_ssc;
	}
	sscMap[entry] = curr_ssc;
}

void AstriMonitor::monitInvalidPackets(BasePacket & pack){
	if(pack.getCRC() < 1)
		num_pack_invalid += 1;
}

