#include <tclap/CmdLine.h>
#include <Astri_Monitor.h>
#include <Astri_Monitor_Configurator.h>


AstriMonitorConfigurator::AstriMonitorConfigurator(){
	setMonitNumPacket(true);
	setMonitByte(true);
	setMonitInvalidPacket(true);
	setMonitLostPacket(true);
	setMonitTime(true);
}


AstriMonitorConfigurator::AstriMonitorConfigurator(int argc, char ** argv){
	try {
		TCLAP::CmdLine cmd("Usage Description WIP", ' ', "1");
		cmd.ignoreUnmatched(true);
		TCLAP::ValueArg<std::string> monitNumPackets("","monitNumPackets","monit the amount of packets",false,"true","[false|true]");
		cmd.add( monitNumPackets );
		TCLAP::ValueArg<std::string> monitLost("","monitLost","monit the amount of packet loss",false,"true","[false|true]");
		cmd.add( monitLost );
		TCLAP::ValueArg<std::string> monitInvalid("","monitInvalid","monit the amount of invalid packets",false,"true","[false|true]");
		cmd.add( monitInvalid );
		TCLAP::ValueArg<std::string> monitTime("","monitTime","monit elapsed time",false,"true","[false|true]");
		cmd.add( monitTime );
		TCLAP::ValueArg<std::string> monitByte("","monitByte","monit byte received",false,"true","[false|true]");
		cmd.add( monitByte );

		cmd.parse( argc, argv );

		setMonitNumPacket( ( monitNumPackets.getValue().compare("true") == 0 ) ? true : false);
		setMonitLostPacket( ( monitLost.getValue().compare("true") == 0 ) ? true : false);
		setMonitInvalidPacket( ( monitInvalid.getValue().compare("true") == 0 ) ? true : false);
		setMonitTime( ( monitTime.getValue().compare("true") == 0 ) ? true : false);
		setMonitByte( ( monitByte.getValue().compare("true") == 0 ) ? true : false);

	}catch (TCLAP::ArgException &e){  // catch any exceptions
		std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl;
		exit(EXIT_FAILURE);
	}
}

bool AstriMonitorConfigurator::isMonitNumPacket(){return this->params["numPackets"].compare("true") == 0 ? true : false;}
bool AstriMonitorConfigurator::isMonitLostPacket(){return this->params["lost"].compare("true") == 0 ? true : false;}
bool AstriMonitorConfigurator::isMonitInvalidPacket(){return this->params["invalid"].compare("true") == 0 ? true : false;}
bool AstriMonitorConfigurator::isMonitTime(){return this->params["time"].compare("true") == 0 ? true : false;}
bool AstriMonitorConfigurator::isMonitByte(){return this->params["byte"].compare("true") == 0 ? true : false;}


BaseMonitor * AstriMonitorConfigurator::createMonitor(){
	return new AstriMonitor(isMonitNumPacket(),isMonitLostPacket(),isMonitInvalidPacket(),isMonitTime(),isMonitByte());
}

AstriMonitor * AstriMonitorConfigurator::createAstriMonitor(){
	return new AstriMonitor(isMonitNumPacket(),isMonitLostPacket(),isMonitInvalidPacket(),isMonitTime(),isMonitByte());
}

void AstriMonitorConfigurator::createAstriMonitor(AstriMonitor & monitor){
	monitor.setMonitTotPacketsFlag(isMonitNumPacket());
	monitor.setmonitPacketLostFlag(isMonitLostPacket());
	monitor.setMonitInvalidPacketsFlag(isMonitInvalidPacket());
	monitor.setMonitTimeFlag(isMonitTime());
	monitor.setMonitByteFlag(isMonitByte());
}


